@@ -7,6 +7,7 @@ use core::fmt::{self, Debug, Display};
 use core::ops::Deref;
 use core::str::from_utf8;
 use serde::{Deserialize, Serialize};
+use std::iter::FromIterator;
 
 /// A collection of Addresses
 #[derive(Debug, Clone, Hash, Ord, PartialOrd, Eq, PartialEq, Serialize, Deserialize)]
@@ -18,27 +19,42 @@ impl AddressSet {
         self.0.iter()
     }
 
-    /// Turn this set into an iterator
-    #[allow(clippy::should_implement_trait)]
-    pub fn into_iter(self) -> impl Iterator<Item = Address> {
-        self.0.into_iter()
-    }
-
     /// Take the first address of the set.
     pub fn first(&self) -> Address {
         self.0.first().cloned().unwrap()
     }
 }
 
+impl IntoIterator for AddressSet {
+    type Item = Address;
+    type IntoIter = std::vec::IntoIter<Self::Item>;
+
+    fn into_iter(self) -> Self::IntoIter {
+        self.0.into_iter()
+    }
+}
+
 impl AsRef<Vec<Address>> for AddressSet {
     fn as_ref(&self) -> &Vec<Address> {
         &self.0
     }
 }
 
+impl<A> FromIterator<A> for AddressSet
+where
+    A: Into<Address>,
+{
+    fn from_iter<T>(iter: T) -> Self
+    where
+        T: IntoIterator<Item = A>,
+    {
+        Self(iter.into_iter().map(Into::into).collect())
+    }
+}
+
 impl<T: Into<Address>> From<Vec<T>> for AddressSet {
     fn from(v: Vec<T>) -> Self {
-        Self(v.into_iter().map(Into::into).collect())
+        v.into_iter().collect()
     }
 }
 
@@ -78,7 +94,47 @@ pub struct Address {
     pub tt: u8,
     inner: Vec<u8>,
 }
+/// An error which is returned when address parsing from string fails
+#[derive(Debug)]
+pub struct AddressParseError {
+    kind: AddressParseErrorKind,
+}
+/// Enum to store the cause of address parsing failure
+#[derive(Debug, Clone, PartialEq, Eq)]
+#[non_exhaustive]
+pub enum AddressParseErrorKind {
+    /// Unable to parse address num in the address string
+    InvalidType(core::num::ParseIntError),
+    /// Address string has more than one '#' separator
+    MultipleSep,
+}
 
+impl AddressParseError {
+    /// Create new instance
+    pub fn new(kind: AddressParseErrorKind) -> Self {
+        Self { kind }
+    }
+    /// Gets the cause of address parsing failure.
+    pub fn kind(&self) -> &AddressParseErrorKind {
+        &self.kind
+    }
+}
+impl Display for AddressParseError {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        match &self.kind {
+            AddressParseErrorKind::InvalidType(e) => {
+                write!(f, "Failed to parse address type: '{}'", e)
+            }
+            AddressParseErrorKind::MultipleSep => {
+                write!(
+                    f,
+                    "Invalid address string: more than one '#' separator found"
+                )
+            }
+        }
+    }
+}
+impl crate::compat::error::Error for AddressParseError {}
 impl Address {
     /// Create a new address from separate type and data parts
     pub fn new<S: Into<String>>(tt: u8, inner: S) -> Self {
@@ -92,38 +148,52 @@ impl Address {
     ///
     /// See type documentation for more detail
     pub fn from_string<S: Into<String>>(s: S) -> Self {
-        let buf: String = s.into();
-        let mut vec: Vec<_> = buf.split('#').collect();
-
-        // If after the split we only have one element, there was no
-        // `#` separator, so the type needs to be implicitly `= 0`
-        let (tt, inner) = if vec.len() == 1 {
-            (0, vec.remove(0).as_bytes().to_vec())
+        match s.into().parse::<Address>() {
+            Ok(a) => a,
+            Err(e) => {
+                panic!("Invalid address string {}", e)
+            }
         }
-        // If after the split we have 2 elements, we extract the type
-        // value from the string, and use the rest as the address
-        else if vec.len() == 2 {
-            let tt = match str::parse(vec.remove(0)) {
-                Ok(tt) => tt,
-                Err(e) => {
-                    panic!("Failed to parse address type: '{}'", e);
-                }
-            };
-
-            (tt, vec.remove(0).as_bytes().to_vec())
-        } else {
-            panic!("Invalid address string: more than one `#` separator found");
-        };
-
-        Self { tt, inner }
     }
-
     /// Generate a random address with a specific type
     pub fn random(tt: u8) -> Self {
         Self { tt, ..random() }
     }
 }
+impl core::str::FromStr for Address {
+    type Err = AddressParseError;
+    /// Parse an address from a string
+    ///
+    /// See type documentation for more detail
+    fn from_str(s: &str) -> Result<Address, Self::Err> {
+        let buf: String = s.into();
+        let mut vec: Vec<_> = buf.split('#').collect();
 
+        // If after the split we only have one element, there was no
+        // `#` separator, so the type needs to be implicitly `= 0`
+        if vec.len() == 1 {
+            Ok(Address {
+                tt: 0,
+                inner: vec.remove(0).as_bytes().to_vec(),
+            })
+        }
+        // If after the split we have 2 elements, we extract the type
+        // value from the string, and use the rest as the address
+        else if vec.len() == 2 {
+            match str::parse(vec.remove(0)) {
+                Ok(tt) => Ok(Address {
+                    tt,
+                    inner: vec.remove(0).as_bytes().to_vec(),
+                }),
+                Err(e) => Err(AddressParseError::new(AddressParseErrorKind::InvalidType(
+                    e,
+                ))),
+            }
+        } else {
+            Err(AddressParseError::new(AddressParseErrorKind::MultipleSep))
+        }
+    }
+}
 impl Display for Address {
     fn fmt<'a>(&'a self, f: &mut fmt::Formatter) -> fmt::Result {
         let inner: &'a str = from_utf8(self.inner.as_slice()).unwrap_or("Invalid UTF-8");
